import configparser
import os

class Config(object):
    ConfMap = {}
    def __init__(self, config_file_path, env='DEV'):
        self.config = configparser.ConfigParser()
        self.config_file_path = config_file_path
        # prevent configparser convert all config keys to lower key
        self.config.optionxform = str
        self.config.read(config_file_path)
        self.env = env
        self.__ConfigMapper()

    def __ConfigMapper(self):
        for section in self.config.sections():
            section_dict = {}
            for key, val in self.config.items(section):
                section_dict[key] = val
                self.ConfMap[str(section)] = section_dict
        
        if self.env is None:
            self.ConfMap = self.ConfMap['DEV']

        if self.env.lower() == 'prod':
            self.ConfMap = self.ConfMap['PROD']
        else:
            self.ConfMap = self.ConfMap['DEV']

    def getValue(self, key):
        value = ''
        try:
            value = self.ConfMap[key]
        except KeyError as key:
            print('Key ' + key + ' did not found in config file')
        return value

    def __resp__(self):
        return f"<Config: {self.config_file_path}>"

    __str__ = __resp__


from .engine import Engine
from .base_model import Model
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import inspect
import os

__version__ = 1

class db_connector(object):
    def __init__(self, config_file_path, show_query=False, env_variable_key='ENV', env_variable_value='production'):
        self.env_variable_key = env_variable_key
        self.env_variable_value = env_variable_value
        self.show_query = show_query
        self.config_file_path = config_file_path
        self.engine = Engine(config_file_path=self.config_file_path,echo=self.show_query, env=self.get_env()).engine
        self.Model = Model
        self.Base = declarative_base()
        __Session = sessionmaker(bind=self.engine, autoflush=False)
        self.session = __Session()
        self.inspector = inspect(self.engine)

    def get_env(self):
        if not self.env_variable_key or not self.env_variable_value:
            return 'DEV'
        if os.environ.get(self.env_variable_key, 'DEV') == self.env_variable_value:
            return 'PROD'
        return 'DEV'
    
    def create_all(self):
        return self.Base.metadata.create_all(self.engine)

    def __repr__(self):
        return f"{self.config_file_path}"
    
    def get_schemas(self):
        return self.inspector.get_schema_names()
    
    def get_all_table_names(self):
        table_names = []
        schemas = self.get_schemas()
        for schema in schemas:
            print("schema: %s" % schema)
            print("*"*50)
            for table_name in self.inspector.get_table_names(schema=schema):
                print(f'table: {table_name}')
                table_names.append(table_name)
        return table_names


from sqlalchemy import Column, Integer, DateTime, func
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy import func
# from . import db

class Model(object):
    __abstract__ = True

    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower()

    id = Column(Integer, primary_key=True, autoincrement=True, nullable=False)
    create_date = Column(DateTime(timezone=True), nullable=False, server_default=func.now())
    update_date = Column(DateTime(timezone=True), nullable=False, server_default=func.now(), onupdate=func.now())

    def to_dict(self):
        """
            converts model to dict
        """
        return {c.name : getattr(self, c.name) for c in self.__table__.columns}

    @classmethod
    def from_dict(cls, dict:dict)->object:
        """
            return model object from a dictionary
        """
        # columns = (dict.get(col, None) for col in dict.keys())
        # print(columns)
        return cls(**dict)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__} object: ID = {self.id}, created at: {self.create_date}"
    __str__ = __repr__

    # @classmethod
    # def all(cls):
    #     objects = db.session.query(cls).all()
    #     return objects
    # @classmethod
    # def first(cls):
    #     return db.session.query(cls).first()
    # @classmethod
    # def filter_by(cls, **kw):
    #     return db.session.query(cls).filter_by(**kw)


from sqlalchemy.sql.schema import Column
from sqlalchemy.sql.sqltypes import Integer, String
import db

class User(db.Model, db.Base):
    name = Column(String(100))
    age = Column(Integer)
    description = Column(String(225))

    def __init__(self, name, age, description):
        self.name = name
        self.age = age
        self.description = description

    def __resp__(self):
        return f"<User {self.name}>"
    
    __str__ = __resp__


from sqlalchemy import create_engine
from .configuration import Config

class Engine:
    """
        create engine to connect to a database based on the config
        by default, engine uses the dev configs from config object
        @param config_file_path (str) : full path to config file
        @param echo (bool) : expose sql query
        @param env (str) : 'DEV' or 'PROD' which are config types in config file
    """
    engine = ''
    def __init__(self, config_file_path, echo=False, env='DEV'):
        self.config = Config(config_file_path, env=env).ConfMap
        self.echo = echo
        self.__engine_db()

    def __engine_db(self):
        """
            create engine, connect to the database
        """
        self.engine = create_engine(
                    f"{self.config['SQL_TYPE_DB']}://{self.config['SQL_USER']}:{self.config['SQL_PASSWORD']}@{self.config['SQL_HOST']}:{self.config['SQL_PORT']}/{self.config['SQL_DATABASE']}",
                    echo=self.echo
                )

    def __repr__(self):
        # return str(self.config)
        return f"<Engine:{self.config['SQL_USER']} - {self.config['SQL_HOST']} - {self.config['SQL_DATABASE']}"

    __str__ = __repr__


from User import User
from db import create_all

# create all tables in database
create_all()

# # import a record into user table
# user = User('do manh dung', '30', 'en france')
# db.session.add(user)
# db.session.commit()

# # get all records in user table
# users = db.session.query(User).all()
# for user in users:
#     print(user.to_dict())

# # create an user object from dict
# # import user object into database
# user = {
#     'name':'DO Manh Dung',
#     'age':30,
#     'description':'Vietnamien'
# }
# obj = User.from_dict(user)
# db.session.add(obj)
# db.session.commit()


